@@ -1,9 +1,9 @@
 #
-# Shorewall 4.4 -- /usr/share/shorewall/Shorewall/Proc.pm
+# Shorewall 5.0 -- /usr/share/shorewall/Shorewall/Proc.pm
 #
 #     This program is under GPL [http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt]
 #
-#     (c) 2007,2008,2009,2010,2011,2012 - Tom Eastep (teastep@shorewall.net)
+#     (c) 2007-2016 - Tom Eastep (teastep@shorewall.net)
 #
 #       Complete documentation is available at http://shorewall.net
 #
@@ -45,7 +45,7 @@ our @EXPORT = qw(
                  setup_log_backend
 		 );
 our @EXPORT_OK = qw( setup_interface_proc );
-our $VERSION = '4.6_4';
+our $VERSION = '4.6_5';
 
 #
 # ARP Filtering
@@ -356,15 +356,35 @@ sub setup_log_backend($) {
 
 	emit( 'progress_message2 "Setting up log backend"',
 	      '',
-	      "if [ -f $file ]; then",
-	      "   if echo $setting > $file; then",
-	      "       progress_message 'Log Backend set to $setting'",
-	      '   else',
-	      "       error_message 'WARNING: Unable to set log backend to $setting'",
-	      '   fi',
-	      'else',
-	      "   error_message 'WARNING: $file does not exist - log backend not set'",
-	      "fi\n" );
+	      "if [ -f $file ]; then"
+	    );
+
+	if ( $setting =~ /ip6?t_log/i ) {
+	    my $alternative = 'nf_log_ipv' . $family;
+
+	    emit( "    setting=$setting",
+		  '',
+		  "    fgrep -q $setting /proc/net/netfilter/nf_log || setting=$alternative",
+		  '',
+		  "    if echo \$setting > $file; then",
+		  '       progress_message "Log Backend set to $setting"',
+		  '   else',
+		  '       error_message "WARNING: Unable to set log backend to $setting"',
+		  '   fi',
+		  'else',
+		  "    error_message 'WARNING: $file does not exist - log backend not set'",
+		  "fi\n"
+		);
+	} else {
+	    emit( "    if echo $setting > $file; then",
+		  "        progress_message 'Log Backend set to $setting'",
+		  '    else',
+		  "        error_message 'WARNING: Unable to set log backend to $setting'",
+		  '    fi',
+		  'else',
+		  "    error_message 'WARNING: $file does not exist - log backend not set'",
+		  "fi\n" );
+	}
     }
 }
 
